package cn.iocoder.yudao.module.bpm.dal.dataobject.oa;

import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.framework.mybatis.core.type.JsonLongSetTypeHandler;
import com.baomidou.mybatisplus.annotation.KeySequence;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.*;

/**
 * BPM OA 工单跟踪记录 DO
 *
 * @author 系统
 */
@TableName(value = "bpm_oa_work_order_track", autoResultMap = true)
@KeySequence("bpm_oa_work_order_track_seq") // 用于 Oracle、PostgreSQL、Kingbase、DB2、H2 数据库的主键自增。如果是 MySQL 等数据库，可不写。
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class BpmOAWorkOrderTrackDO extends BaseDO {

    /**
     * 跟踪记录ID
     */
    @TableId
    private Long id;
    
    /**
     * 工单ID
     */
    private Long workOrderId;
    
    /**
     * 操作人ID
     */
    private Long operatorId;
    
    /**
     * 操作人姓名
     */
    private String operatorName;
    
    /**
     * 操作类型
     * 
     * 枚举值：
     * - CREATE: 创建工单
     * - ASSIGN: 分配工单
     * - PROCESS: 处理中
     * - COMPLETE: 完成工单
     * - CANCEL: 取消工单
     * - COMMENT: 添加备注
     */
    private String operationType;
    
    /**
     * 操作内容/备注
     */
    private String content;
    
    /**
     * 附件文件列表，JSON格式
     */
    @TableField(typeHandler = JsonLongSetTypeHandler.class)
    private String attachments;

} 