package cn.iocoder.yudao.module.bpm.service.oa;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.bpm.api.oa.vo.loan.BpmOALoanSumDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.loan.BpmOALoanCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.loan.BpmOALoanPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.loan.BpmOAReturnVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOALoanDO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 生产借支 Service 接口
 *
 * @author 符溶馨
 */
public interface BpmOALoanService {

    /**
     * 创建借支申请
     *
     * @param userId      用户编号
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createLoan(Long userId, @Valid BpmOALoanCreateReqVO createReqVO);

    /**
     * 更新借支申请的状态
     *
     * @param processInstanceId 流程实例编号
     * @param id                   id
     * @param result               结果
     */
    void updateLoanResult(String processInstanceId, Long id, Integer result);

    /**
     * 获得借支申请
     *
     * @param id 编号
     * @return 加班申请
     */
    BpmOALoanDO getLoan(Long id);

    /**
     * 获得指定的借支申请
     *
     * @param processInstanceId 流程实例编号
     * @return 借支申请
     */
    BpmOALoanDO getByProcessInstanceId(String processInstanceId);

    /**
     * 获得指定用户的借支分页列表
     * @param pageReqVO 分页参数
     * @return 借支列表
     */
    PageResult<BpmOALoanDO> getListByStaffId(BpmOALoanPageReqVO pageReqVO);

    /**
     * 获取员工当月需抵扣的借支金额
     * @param staffId 员工编号集合
     * @return 借支金额统计
     */
    List<BpmOALoanSumDTO> getListByStaffId(Collection<Long> staffId, String month);

    /**
     * 获取员工还款分页列表
     * @param pageReqVO 分页信息
     * @return 还款分页列表
     */
    PageResult<BpmOAReturnVO> getReturnList(BpmOALoanPageReqVO pageReqVO);
}
