package cn.iocoder.yudao.module.bpm.service.oa;

import cn.iocoder.yudao.framework.common.pojo.UploadUserFile;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.petition.BpmOAPetitionCreateReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAPetitionDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.BpmOAPetitionMapper;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.bpm.service.task.BpmHistoryProcessInstanceService;
import cn.iocoder.yudao.module.bpm.service.task.BpmProcessInstanceService;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_PETITION_NOT_EXISTS;

/**
 * OA 签呈申请 Service 实现类
 *
 * @author 符溶馨
 */
@Service
@Validated
public class BpmOAPetitionServiceImpl extends BpmOABaseService implements BpmOAPetitionService{

    /**
     * OA 签呈对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_petition_2";

    @Resource
    private BpmOAPetitionMapper petitionMapper;

    @Resource
    private BpmProcessInstanceService processInstanceService;

    @Resource
    private BpmHistoryProcessInstanceService historyProcessInstanceService;

    @Override
    public Long createPetition(Long userId, BpmOAPetitionCreateReqVO createReqVO) {

        //插入OA 签呈申请
        BpmOAPetitionDO petitionDO = BeanUtils.toBean(createReqVO, BpmOAPetitionDO.class).setUserId(userId)
                .setResult(BpmProcessInstanceResultEnum.PROCESS.getResult());
        petitionMapper.insert(petitionDO);

        // 发起 BPM 流程
        Map<String, Object> processInstanceVariables = new HashMap<>();
        String processInstanceId = processInstanceService.createProcessInstance(userId,
                new BpmProcessInstanceCreateReqDTO().setProcessDefinitionKey(PROCESS_KEY)
                        .setVariables(processInstanceVariables).setBusinessKey(String.valueOf(petitionDO.getId())));

        // 将工作流的编号，更新到 OA 加班单中
        petitionMapper.updateById(new BpmOAPetitionDO().setId(petitionDO.getId()).setProcessInstanceId(processInstanceId));

        // 判断是否为重新发起的流程
        if (createReqVO.getProcessInstanceId() != null && createReqVO.getResult() == 3) {

            historyProcessInstanceService.createHistoryProcessInstance(processInstanceId, createReqVO.getProcessInstanceId());
        }

        List<UploadUserFile> fileItems = createReqVO.getFileItems() ;
        //这里的逻辑，如果fileItems不为空，且有数据，那么说明是上传了附件的，则需要更工作流文件表对应的实例Id
        if (fileItems != null && !fileItems.isEmpty()) {
            uploadBpmFileProcessInstanceId(processInstanceId,fileItems) ;
        }
        return petitionDO.getId();
    }

    @Override
    public void updatePetitionServiceResult(Long id, Integer result) {

        validateLeaveExists(id);
        petitionMapper.updateById(new BpmOAPetitionDO().setId(id).setResult(result));
    }

    @Override
    public BpmOAPetitionDO getPetition(Long id) {

        return petitionMapper.selectById(id);
    }

    @Override
    public BpmOAPetitionDO getByProcessInstanceId(String processInstanceId) {

        return petitionMapper.selectOne(BpmOAPetitionDO::getProcessInstanceId, processInstanceId);
    }

    private void validateLeaveExists(Long id) {
        if (petitionMapper.selectById(id) == null) {
            throw exception(OA_PETITION_NOT_EXISTS);
        }
    }
}
