package cn.iocoder.yudao.module.system.dal.dataobject.project;

import cn.iocoder.yudao.framework.mybatis.core.type.JsonLongSetTypeHandler;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import lombok.*;

import java.time.LocalDate;
import java.util.*;
import java.time.LocalDateTime;
import java.time.LocalDateTime;
import com.baomidou.mybatisplus.annotation.*;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;

/**
 * 项目管理 DO
 *
 * @author 符溶馨
 */
@TableName(value = "system_project", autoResultMap = true)
@KeySequence("system_project_seq") // 用于 Oracle、PostgreSQL、Kingbase、DB2、H2 数据库的主键自增。如果是 MySQL 等数据库，可不写。
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class ProjectDO extends BaseDO {

    /**
     * 项目管理表单主键
     */
    @TableId
    private Long id;
    /**
     * 项目编号
     */
    private String projectNo;
    /**
     * 项目类型 | 字典值参考system_project_type
     */
    private Integer type;
    /**
     * 项目名称
     */
    private String name;
    /**
     * 责任部门
     */
    private Long responsibleDept;
    /**
     * 参与部门集合
     */
    @TableField(typeHandler = JsonLongSetTypeHandler.class)
    private Set<Long> participationDept;
    /**
     * 责任人用户编号
     */
    private Long directorUserId;
    /**
     * 项目开始时间
     */
    private LocalDate startDate;
    /**
     * 项目结束时间
     */
    private LocalDate endDate;
    /**
     * 是否长期 | 0否 1是
     */
    private Integer isLongTerm;
    /**
     * 项目预算
     */
    private Integer projectBudget;
    /**
     * 项目内容
     */
    private String content;
    /**
     * 项目人员用户编号集合
     */
    @TableField(typeHandler = JsonLongSetTypeHandler.class)
    private Set<Long> staff;
    /**
     * 项目额外属性
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private Map<String, Object> dynamicAttribute;
    /**
     * 项目状态 | 1 进行中 2已完成
     */
    private Integer status;

}