package cn.iocoder.yudao.module.system.service.assets;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.api.assets.dto.SupplierAssetAllocationDTO;
import cn.iocoder.yudao.module.system.controller.admin.assets.vo.AssetsPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.assets.vo.AssetsSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.assets.AssetsDO;
import com.baomidou.mybatisplus.extension.service.IService;

import javax.validation.Valid;
import java.util.List;

/**
 * 资产 Service 接口
 *
 * @author 艾楷
 */
public interface AssetsService extends IService<AssetsDO> {

    /**
     * 创建资产
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createAssets(@Valid AssetsSaveReqVO createReqVO);

    /**
     * 更新资产
     *
     * @param updateReqVO 更新信息
     */
    void updateAssets(@Valid AssetsSaveReqVO updateReqVO);

    /**
     * 删除资产
     *
     * @param id 编号
     */
    void deleteAssets(Long id);

    /**
     * 获得资产
     *
     * @param id 编号
     * @return 资产
     */
    AssetsDO getAssets(Long id);

    /**
     * 获得资产分页
     *
     * @param pageReqVO 分页查询
     * @return 资产分页
     */
    PageResult<AssetsDO> getAssetsPage(AssetsPageReqVO pageReqVO);

    /**
     * 供应商资产分配
     *
     * @param dtos
     */
    void supplierAssetAllocation(List<SupplierAssetAllocationDTO> dtos);
}
