package cn.iocoder.yudao.module.system.service.holiday.holidaysettingrange;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.holiday.holidaysettingrange.vo.HolidaySettingRangePageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.holiday.holidaysettingrange.vo.HolidaySettingRangeSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.holiday.holidaysettingrange.HolidaySettingRangeDO;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 假期规则人员范围 Service 接口
 *
 * @author 艾楷
 */
public interface HolidaySettingRangeService {

    /**
     * 创建假期规则人员范围
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createHolidaySettingRange(@Valid HolidaySettingRangeSaveReqVO createReqVO);

    /**
     * 更新假期规则人员范围
     *
     * @param updateReqVO 更新信息
     */
    void updateHolidaySettingRange(@Valid HolidaySettingRangeSaveReqVO updateReqVO);

    /**
     * 删除假期规则人员范围
     *
     * @param id 编号
     */
    void deleteHolidaySettingRange(Long id);

    /**
     * 获得假期规则人员范围
     *
     * @param id 编号
     * @return 假期规则人员范围
     */
    HolidaySettingRangeDO getHolidaySettingRange(Long id);

    /**
     * 获得假期规则人员范围分页
     *
     * @param pageReqVO 分页查询
     * @return 假期规则人员范围分页
     */
    PageResult<HolidaySettingRangeDO> getHolidaySettingRangePage(HolidaySettingRangePageReqVO pageReqVO);

    /**
     * 通过设置id获取假期规则人员范围
     *
     * @param settingId
     * @return
     */
    List<HolidaySettingRangeDO> selectBySettingId(Long settingId);

    /**
     * 通过范围获取用户列表
     *
     * @param holidaySettingRangeDOS
     * @return
     */
    List<AdminUserDO> getUsersByRange(List<HolidaySettingRangeDO> holidaySettingRangeDOS);

    /**
     * 根据假期设置id获取列表
     *
     * @return
     */
    List<HolidaySettingRangeDO> getListByHolidaySettingId(Long holidaySettingId);

    /**
     * 根据假期设置ids获取列表
     *
     * @param holidaySettingIds
     * @return
     */
    List<HolidaySettingRangeDO> getListByHolidaySettingIds(List<Long> holidaySettingIds);

    /**
     * 根据考勤组id获取当前考勤组所在的假期
     *
     * @param groupId
     * @return
     */
    List<HolidaySettingRangeDO> getByGroupId(Long groupId);

    /**
     * 根据考勤组id 获取和考勤组关联的假期ids
     *
     * @param groupId
     * @return
     */
    List<Long> getHolidaySettingIdsByGroupId(Long groupId);
}
